MODULE ctl_one;

<*/Resource:ctl_one.res*>

FROM WIN32    IMPORT HWND,UINT,WPARAM,LPARAM,BOOL,INT_PTR;
FROM WINUSER  IMPORT WM_CLOSE,WM_COMMAND,WM_INITDIALOG,
                     LOWORD,HIWORD,
                     MessageBox,MB_OK,
                     EndDialog,
                     GetWindowTextLength,GetDlgItem,GetDlgItemInt,GetDlgItemText,
                     SendMessage,SendDlgItemMessage,SetDlgItemInt,SetDlgItemText,
                     LB_ERR,LB_ADDSTRING,LB_SETITEMDATA,LB_GETSELCOUNT,
                     LB_GETSELITEMS,LB_DELETESTRING,LB_RESETCONTENT,
                     LB_GETITEMDATA,LBN_SELCHANGE;
FROM WINX     IMPORT Instance,DialogBoxId;

FROM SYSTEM   IMPORT CAST,FUNC,ADR;
FROM Storage  IMPORT ALLOCATE,DEALLOCATE;

FROM resource IMPORT IDD_MAIN,IDC_TEXT,IDC_NUMBER,IDC_LIST,IDC_ADD,
                     IDC_CLEAR,IDC_REMOVE,IDC_SHOWCOUNT;



PROCEDURE Add(hwnd : HWND);
VAR bSuccess : BOOL;
    nTimes   : INTEGER;
    len      : INTEGER;
    i,index  : INTEGER;
    buf      : POINTER TO ARRAY OF CHAR;
BEGIN
    (* When somebody clicks the Add button, first we get the number of
      they entered *)
    nTimes := GetDlgItemInt(hwnd, IDC_NUMBER, bSuccess, FALSE);
    (* If Success we get the string they entered
       First we need to find out how long it is so that we can
       allocate some memory *)
    IF bSuccess THEN
       len := GetWindowTextLength(GetDlgItem(hwnd, IDC_TEXT));
       IF len > 0 THEN
         (* Now we allocate, and get the string into our buffer *)
         NEW(buf,len + 1);
         FUNC GetDlgItemText(hwnd, IDC_TEXT, buf^, len + 1);
         (* Now we add the string to the list box however many times
            the user asked us to. *)
         FOR i:=0 TO nTimes-1 DO
           index := SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING,
                                       0, CAST(LPARAM,buf));
           (* Here we are associating the value nTimes with the item
              just for the heck of it, we'll use it to display later.
              Normally you would put some more useful data here, such
              as a pointer. *)
           FUNC SendDlgItemMessage(hwnd, IDC_LIST, LB_SETITEMDATA, index, nTimes);
         END;
         DISPOSE(buf);
       ELSE
         FUNC MessageBox(hwnd, "You didn't enter anything!", "Warning", MB_OK);
       END;
    ELSE
       FUNC MessageBox(hwnd, "Couldn't translate that number :(", "Warning", MB_OK);
    END;
END Add;


PROCEDURE Remove(hwnd : HWND);
VAR hList : HWND;
    count : INTEGER;
    i     : INTEGER;
    buf   : POINTER TO ARRAY OF INTEGER;
BEGIN
    (* When the user clicks the Remove button, we first get the number
       of selected items *)

    hList := GetDlgItem(hwnd, IDC_LIST);
    count := SendMessage(hList, LB_GETSELCOUNT, 0, 0);
    IF count # LB_ERR THEN
      IF count # 0 THEN
        (* And then allocate room to store the list of selected items. *)

        NEW(buf,count);
        FUNC SendMessage(hList, LB_GETSELITEMS, count, CAST(LPARAM,buf));

        (* Now we loop through the list and remove each item that was selected.
           WARNING!!!
           We loop backwards, because if we removed items
           from top to bottom, it would change the indexes of the other
           items!!!   *)

        FOR i:=count-1 TO 0 BY -1 DO
           FUNC SendMessage(hList, LB_DELETESTRING, buf^[i], 0);
        END;
        DISPOSE(buf);
      ELSE
        FUNC MessageBox(hwnd, "No items selected.", "Warning", MB_OK);
      END;
    ELSE
      FUNC MessageBox(hwnd, "Error counting items :(", "Warning", MB_OK);
    END;
END Remove;

PROCEDURE Clear(hwnd : HWND);
BEGIN
    FUNC SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
END Clear;

PROCEDURE List(hwnd : HWND);
VAR hList : HWND;
    count : INTEGER;
    index : INTEGER;
    err   : INTEGER;
    data  : INTEGER;
BEGIN
    (* Get the number of items selected. *)

    hList := GetDlgItem(hwnd, IDC_LIST);
    count := SendMessage(hList, LB_GETSELCOUNT, 0, 0);
    IF count # LB_ERR THEN
      (* We only want to continue if one and only one item is selected.  *)
      IF count = 1 THEN
        (* Since we know ahead of time we're only getting one
           index, there's no need to allocate an array. *)
        err := SendMessage(hList, LB_GETSELITEMS, 1, CAST(LPARAM,ADR(index)));
        IF err # LB_ERR THEN
          (* Get the data we associated with the item above
            (the number of times it was added) *)
          data := SendMessage(hList, LB_GETITEMDATA, index, 0);
          FUNC SetDlgItemInt(hwnd, IDC_SHOWCOUNT, data, FALSE);
        ELSE
          FUNC MessageBox(hwnd, "Error getting selected item :(", "Warning", MB_OK);
        END;
      ELSE
        (* No items selected, or more than one
           Either way, we aren't going to process this. *)
        FUNC SetDlgItemText(hwnd, IDC_SHOWCOUNT, "-");
      END;
    ELSE
      FUNC MessageBox(hwnd, "Error counting items :(", "Warning", MB_OK);
    END;
END List;


(* *** Exported dialog procedure *** *)
PROCEDURE DlgProc(hwnd : HWND; msg : UINT;
                       wParam : WPARAM; lParam : LPARAM) : INT_PTR [EXPORT, OSCall];
BEGIN
    CASE msg OF
    | WM_INITDIALOG : (* This is where we set up the dialog box,
                         and initialise any default values *)
                      FUNC SetDlgItemText(hwnd, IDC_TEXT, "This is a string");
                      FUNC SetDlgItemInt(hwnd, IDC_NUMBER, 5, FALSE);
    | WM_CLOSE      : FUNC EndDialog(hwnd, 0);
    | WM_COMMAND    :
      CASE LOWORD(wParam) OF
      | IDC_ADD     : Add(hwnd);
      | IDC_REMOVE  : Remove(hwnd);
      | IDC_CLEAR   : Clear(hwnd);
      | IDC_LIST    : IF HIWORD(wParam) = LBN_SELCHANGE THEN
                        List(hwnd);
                      END;
      ELSE
      END;
    ELSE RETURN 0;
    END;
    RETURN 1;
END DlgProc;


BEGIN
    FUNC DialogBoxId(Instance, IDD_MAIN, NIL, DlgProc);
END ctl_one.
